@@ -1,4 +1,3 @@
-
 module Parser --(parseAG)
 where
 import Data.Maybe
@@ -7,7 +6,6 @@ import UU.Parsing.Machine(RealParser(..),RealRecogn(..),anaDynE,mkPR)
 import ConcreteSyntax
 import CommonTypes
 import Patterns
-import Rules
 import UU.Pretty(text,PP_Doc,empty,(>-<))
 import TokenDef
 import List (intersperse)
@@ -22,18 +20,26 @@ import UU.Scanner.GenTokenParser
 import UU.Scanner.Position
 import UU.Scanner.TokenShow()
 import System.Directory
+
+
 type AGParser = AnaParser Input  Pair Token Pos
 
 pIdentifier, pIdentifierU :: AGParser Identifier
 pIdentifierU = uncurry Ident <$> pConidPos
 pIdentifier   = uncurry Ident <$> pVaridPos
 
+
 parseAG :: [FilePath] -> String -> IO (AG,[Message Token Pos])
 parseAG  searchPath file 
-              = do (es,_,mesg) <- parseFile searchPath file
+              = do (es,_,_,mesg) <- parseFile searchPath file
                    return (AG es, mesg)
 
-parseFile :: [FilePath] -> String -> IO  ([Elem],[String],[Message Token Pos ])
+depsAG :: [FilePath] -> String -> IO ([String], [Message Token Pos])
+depsAG searchPath file
+  = do (_,_,fs,mesgs) <- parseFile searchPath file
+       return (fs, mesgs)
+
+parseFile :: [FilePath] -> String -> IO  ([Elem],[String],[String],[Message Token Pos ])
 parseFile searchPath file 
                = do txt <- readFile file
                     let litMode = ".lag" `isSuffixOf` file
@@ -42,13 +48,15 @@ parseFile searchPath file
                         tokens       = input (initPos file) text
 
                         steps = parse pElemsFiles tokens
-                        stop (_,fs,_) = null fs
-                        cont (es,fs,msg) = do files <- mapM (resolveFile searchPath) fs
-                                              res <- mapM (parseFile searchPath) files
-                                              let (ess,fss,msgs) = unzip3 res
-                                              return (es ++ concat ess, concat fss, msg ++ concat msgs)
+                        stop (_,fs,_,_) = null fs
+                        cont (es,fs,allfs,msg)
+                          = do files <- mapM (resolveFile searchPath) fs
+                               res <- mapM (parseFile searchPath) files
+                               let (ess,fss,allfss, msgs) = unzip4 res
+                               return (es ++ concat ess, concat fss, concat allfss ++ allfs, msg ++ concat msgs)
                     let (Pair (es,fls) _ ,mesg) = evalStepsMessages steps
-                    loop stop cont (es,files++fls,mesg)
+                    let allfs = files ++ fls
+                    loopp stop cont (es,allfs,allfs,mesg)
 
 resolveFile :: [FilePath] -> FilePath -> IO FilePath
 resolveFile path fname = search (path ++ ["."])                                                  
@@ -75,18 +83,11 @@ evalStepsMessages steps = case steps of
   Best _   rest   _   -> evalStepsMessages rest
   NoMoreSteps v       -> (v,[])
 
-showMessage (Msg expect pos action) = 
-                        "\nParse error: " ++ show pos ++"\n" ++
-                        "Expecting " ++ show expect ++ "\n"++
-                        "Repaired by " ++ actionMessage ++ "\n" 
-    where actionMessage =  case action of
-                             Insert s -> "inserting: " ++ show s
-                             Delete s -> "deleting: "  ++ show s
-                             Other ms -> ms
-loop ::(a->Bool) -> (a->IO a) -> a ->  IO a
-loop pred cont x | pred x = return x
-                 | otherwise = do x' <- cont x
-                                  loop pred cont x'
+loopp ::(a->Bool) -> (a->IO a) -> a ->  IO a
+loopp pred cont x | pred x = return x
+                  | otherwise = do x' <- cont x
+                                   loopp pred cont x'
+                                  
 pElemsFiles :: AGParser ([Elem],[String])
 pElemsFiles = pFoldr (($),([],[])) pElem'
    where pElem' =  addElem <$> pElem
@@ -128,6 +129,9 @@ pNontSet = set0
             <|> All <$ pStar
             <|> pParens set0
 
+pNames :: AGParser [Name]
+pNames = pList1 pIdentifier
+
 pAG  :: AGParser AG
 pAG  = AG <$> pElems
 
@@ -143,7 +147,7 @@ pComplexType =  List <$> pBracks pType
                         ]
 pElem :: AGParser Elem
 pElem =  Data <$> pDATA
-              <*> pIdentifierU
+              <*> pNontSet
               <*> pOptAttrs
               <*> pAlts
              <*> pSucceed False
@@ -155,7 +159,7 @@ pElem =  Data <$> pDATA
               <*  pEquals
               <*> pComplexType
      <|> Sem  <$> pSEM
-              <*> pIdentifierU
+              <*> pNontSet
               <*> pOptAttrs
               <*> pSemAlts
      <|> Set  <$> pSET
@@ -170,7 +174,15 @@ pElem =  Data <$> pDATA
      <|> Wrapper
               <$> pWRAPPER
               <*> pNontSet
-   <|> codeBlock <$> (pIdentifier <|> pSucceed (Ident "" noPos)) <*> pCodeBlock <?> "a statement"
+     <|> Pragma
+              <$> pPRAGMA
+              <*> pNames
+     <|> Module
+              <$> pMODULE
+              <*> pCodescrap'
+              <*> pCodescrap'
+              <*> pCodescrap'
+     <|> codeBlock <$> (pIdentifier <|> pSucceed (Ident "" noPos)) <*> pCodeBlock <?> "a statement"
            where codeBlock nm (txt,pos) = Txt pos nm (lines txt)
 
 -- Insertion is expensive for pCodeBlock in order to prevent infinite inserts.
@@ -193,7 +205,7 @@ pType =  NT <$> pIdentifierU
 
 
 pInhAttrNames :: AGParser AttrNames
-pInhAttrNames   = (\vs tp -> map (\v -> (v,tp,("",""))) vs)
+pInhAttrNames   = (\vs tp -> map (\v -> (v,tp,("","",""))) vs)
                   <$> pIdentifiers <*  pColon <*> pType <?> "attribute declarations"
 
 pIdentifiers :: AGParser [Identifier]
@@ -204,11 +216,11 @@ pAttrNames :: AGParser AttrNames
 pAttrNames = (\vs use tp -> map (\v -> (v,tp,use)) vs)
              <$> pIdentifiers <*> pUse <* pColon <*> pType <?> "attribute declarations"
 
-pUse :: AGParser (String,String)
-pUse = ((,) <$ pUSE <*> pCodescrap'  <*> pCodescrap')` opt` ("","") <?> "USE declaration"
+pUse :: AGParser (String,String,String)
+pUse = (  (\u x y->(x,y,show u)) <$> pUSE <*> pCodescrap'  <*> pCodescrap')` opt` ("","","") <?> "USE declaration"
 
 pAlt :: AGParser Alt
-pAlt =  Alt <$> pBar <*> pIdentifierU <*> pFields <?> "a datatype alternative"
+pAlt =  Alt <$> pBar <*> pSimpleConstructorSet <*> pFields <?> "a datatype alternative"
 
 pAlts :: AGParser Alts
 pAlts =  pList_ng pAlt <?> "datatype alternatives"
@@ -228,7 +240,11 @@ mklower []     = []
 pSemAlt :: AGParser SemAlt
 pSemAlt  = SemAlt
           <$> pBar <*> pConstructorSet <*> pSemDefs <?> "SEM alternative"
---  where makeSemAlts p cs defs = [SemAlt p c defs | c <- cs ]
+
+pSimpleConstructorSet :: AGParser ConstructorSet
+pSimpleConstructorSet =  CName <$> pIdentifierU
+                     <|> CAll  <$  pStar
+                     <|> pParens pConstructorSet
 
 pConstructorSet :: AGParser ConstructorSet
 pConstructorSet =  pChainl (CDifference <$ pMinus) term2
@@ -241,10 +257,16 @@ pSemAlts :: AGParser SemAlts
 pSemAlts =  pList pSemAlt <?> "SEM alternatives"
 
 pFieldIdentifier =  pIdentifier 
-                <|> Ident "lhs" <$> pLHS 
-                <|> Ident "loc" <$> pLOC
+                <|> Ident "lhs"  <$> pLHS 
+                <|> Ident "loc"  <$> pLOC
+                <|> Ident "inst" <$> pINST
 
+pSemDef :: AGParser [SemDef]
 pSemDef = (\x fs -> map ($ x) fs)<$> pFieldIdentifier <*> pList1 pAttrDef
+      <|>                            pLOC              *> pList1 pLocDecl
+      <|>                            pINST             *> pList1 pInstDecl
+      <|>  pSEMPRAGMA *> pList1 (SemPragma <$> pNames)
+      <|> (\a b -> [AttrOrderBefore a b]) <$> pAttr <* pSmaller <*> pAttr
       <|> (\pat owrt exp -> [Def (pat ()) exp owrt]) <$> pPattern (const <$> pAttr) <*> pAssign <*> pExpr
  
 pAttr = (,) <$> pFieldIdentifier <* pDot <*> pIdentifier
@@ -253,8 +275,25 @@ pAttrDef :: AGParser (Name -> SemDef)
 pAttrDef = (\pat owrt exp fld -> Def (pat fld) exp owrt)
            <$ pDot <*> pattern <*> pAssign <*> pExpr
   where pattern =  pPattern pVar
-               <|> (\a fld -> Alias fld a (Underscore noPos)) <$> pIdentifier
- 
+               <|> (\a fld -> Alias fld a (Underscore noPos) []) <$> pIdentifier
+
+
+nl2sp :: Char -> Char
+nl2sp '\n' = ' '
+nl2sp '\r' = ' '
+nl2sp x = x
+
+pLocDecl :: AGParser SemDef
+pLocDecl =   (\ident tp -> TypeDef ident tp)
+              <$ pDot <*> pIdentifier <* pColon <*> pLocType
+
+pLocType = (Haskell . getName) <$> pIdentifierU
+       <|> Haskell <$> pCodescrap'  <?> "a type"
+
+pInstDecl :: AGParser SemDef
+pInstDecl = (\ident tp -> TypeDef ident tp)
+             <$ pDot <*> pIdentifier <* pColon <*> (NT <$> pIdentifierU)
+
 pSemDefs :: AGParser SemDefs
 pSemDefs =  concat <$> pList_ng pSemDef  <?> "attribute rules"
 
@@ -279,7 +318,7 @@ pPattern pvar = pPattern2 where
                <|> pPattern1 <?> "a pattern"
   pPattern1 =  pvariable 
            <|> pPattern2
-  pvariable = (\var pat a -> case var a of (fld,att) -> Alias fld att (pat a)) 
+  pvariable = (\var pat a -> case var a of (fld,att) -> Alias fld att (pat a) []) 
            <$> pvar <*> ((pAt *> pPattern1) `opt` const (Underscore noPos)) 
   pPattern2 = (mkTuple <$> pOParenPos <*> pListSep pComma pPattern0 <* pCParen )
           <|> (const . Underscore) <$> pUScore <?> "a pattern"
@@ -295,12 +334,14 @@ pCodescrap ::  AGParser (String,Pos)
 pCodescrap   = pCodeBlock
 
 pSEM, pATTR, pDATA, pUSE, pLOC,pINCLUDE, pTYPE, pEquals, pColonEquals,
-      pBar, pColon, pLHS,pSET,pDERIVING,pMinus,pIntersect,pArrow,
-      pDot, pUScore, pEXT,pAt,pStar, pWRAPPER, pMAYBE
+      pBar, pColon, pLHS,pINST,pSET,pDERIVING,pMinus,pIntersect,pArrow,
+      pDot, pUScore, pEXT,pAt,pStar, pSmaller, pWRAPPER, pMAYBE, pMODULE
       :: AGParser  Pos
 pSET         = pCostReserved 90 "SET"     <?> "SET"
 pDERIVING    = pCostReserved 90 "DERIVING"<?> "DERIVING"
 pWRAPPER     = pCostReserved 90 "WRAPPER" <?> "WRAPPER"
+pPRAGMA      = pCostReserved 90 "PRAGMA"  <?> "PRAGMA"
+pSEMPRAGMA   = pCostReserved 90 "SEMPRAGMA" <?> "SEMPRAGMA"
 pDATA        = pCostReserved 90 "DATA"    <?> "DATA"
 pEXT         = pCostReserved 90 "EXT"     <?> "EXT"
 pATTR        = pCostReserved 90 "ATTR"    <?> "ATTR"
@@ -311,6 +352,7 @@ pMAYBE       = pCostReserved 5  "MAYBE"   <?> "MAYBE"
 pUSE         = pCostReserved 5  "USE"     <?> "USE"
 pLOC         = pCostReserved 5  "loc"     <?> "loc"
 pLHS         = pCostReserved 5  "lhs"     <?> "loc"
+pINST        = pCostReserved 5  "inst"    <?> "inst"
 pAt          = pCostReserved 5  "@"       <?> "@"
 pDot         = pCostReserved 5  "."       <?> "."
 pUScore      = pCostReserved 5  "_"       <?> "_"
@@ -322,5 +364,6 @@ pIntersect   = pCostReserved 5  "/\\"     <?> "/\\"
 pMinus       = pCostReserved 5  "-"       <?> "-"
 pArrow       = pCostReserved 5  "->"      <?> "->"
 pStar        = pCostReserved 5  "*"       <?> "*"
-
+pSmaller     = pCostReserved 5  "<"       <?> "<"
+pMODULE      = pCostReserved 5  "MODULE"  <?> "MODULE"
 
