@@ -42,15 +42,16 @@ scan p ('-':'-':xs)              = let (com,rest) = span (/= '\n') xs
 scan p ('{':'-':xs)              = advc' 2 p (ncomment scan) xs
 scan p ('{'    :xs)              = advc' 1 p codescrap xs
 scan p ('\CR':xs)                = case xs of
-                                    '\LF':ys -> newl' p scan ys --ms newline
-                                    _        -> newl' p scan xs --mac newline
-scan p ('\LF':xs)                =  newl' p scan xs             --unix newline
+                                    '\LF':ys -> newl' p scanBeginOfLine ys --ms newline
+                                    _        -> newl' p scanBeginOfLine xs --mac newline
+scan p ('\LF':xs)                =  newl' p scanBeginOfLine xs             --unix newline
 scan p (x:xs) | isSpace x        = updPos'  x p scan  xs
 scan p xs = Just (scan' xs)
   where scan' ('.' :rs)          = (reserved "." p, advc 1 p, rs)
         scan' ('@' :rs)          = (reserved "@" p, advc 1 p, rs)
         scan' (',' :rs)          = (reserved "," p, advc 1 p, rs)
         scan' ('_' :rs)          = (reserved "_" p, advc 1 p, rs)
+        scan' ('<' :rs)          = (reserved "<" p, advc 1 p, rs)
         scan' ('[' :rs)          = (reserved "[" p, advc 1 p, rs)
         scan' (']' :rs)          = (reserved "]" p, advc 1 p, rs)
         scan' ('(' :rs)          = (reserved "(" p, advc 1 p, rs)
@@ -88,10 +89,31 @@ scan p xs = Just (scan' xs)
                                    in (tok p, advc (length var+1) p,rest)
                      | otherwise = (errToken ("unexpected character " ++ show x) p, advc 1 p, rs)
 
+scanBeginOfLine :: Lexer Token
+scanBeginOfLine p ('{' : '-' : ' ' : 'L' : 'I' : 'N' : 'E' : ' ' : xs)
+  | isOkBegin rs && isOkEnd rs'
+      = scan (advc (8 + length r + 2 + length s + 4) p') (drop 4 rs')
+  | otherwise
+      = Just (errToken ("Invalid LINE pragma: " ++ show r) p, advc 8 p, xs)
+  where
+    (r,rs)   = span isDigit xs
+    (s, rs') = span (/= '"') (drop 2 rs)
+    p' = Pos (read r - 1) (column p) s    -- LINE pragma indicates the line number of the /next/ line!
+    
+    isOkBegin (' ' : '"' : _) = True
+    isOkBegin _               = False
+    
+    isOkEnd ('"' : ' ' : '-' : '}' : _) = True
+    isOkEnd _         = False
+scanBeginOfLine p xs
+  = scan p xs
+ 
+
 ident = span isValid
  where isValid x = isAlphaNum x || x =='_' || x == '\''
-keywords = [ "DATA", "EXT", "ATTR", "SEM","TYPE", "USE", "loc","lhs", "INCLUDE"
-           ,"SET","DERIVING","FOR", "WRAPPER", "MAYBE"
+keywords = [ "DATA", "EXT", "ATTR", "SEM","TYPE", "USE", "loc","lhs", "inst", "first", "last", "INCLUDE"
+           , "SET","DERIVING","FOR", "WRAPPER", "MAYBE"
+           , "PRAGMA", "SEMPRAGMA", "MODULE"
            ]
 
 ncomment c p ('-':'}':xs) = advc' 2 p c  xs
@@ -131,7 +153,9 @@ scanLit xs = (fs, foldr insNL (const "") codeLns 1)
 
         getBlock = getLines . dropWhile comment
         getLines [] = ([],[],[])
-        getLines ((n,l):ls) | "\\begin{Code}" `isPrefixOf` l = let (lns,rest) = codelines ls
+        getLines ((n,l):ls) | "\\begin{code}" `isPrefixOf` l = let (lns,rest) = codelines ls
+                                                               in ([],lns,rest)
+                            | "\\begin{Code}" `isPrefixOf` l = let (lns,rest) = codeLines ls
                                                                in ([],lns,rest)
                             | "\\IN{" `isPrefixOf` l        =
                                      let name = getName l
@@ -148,16 +172,20 @@ breakLine xs = case xs of
                                 '\LF' : zs -> ([],zs) 
                                 _          -> ([],ys)
                 '\LF' : ys -> ([], ys)
-                '\n'  : ys -> ([], ys)
                 x     : ys -> let (l,s) = breakLine ys
                               in (x:l,s)
                 []         -> ([],[])
  
-codelines [] = error "Unterminated code block"
-codelines ((n,l):ls) | "\\end{Code}" `isPrefixOf` l = ([],ls)
+codelines [] = error "Unterminated literate code block"
+codelines ((n,l):ls) | "\\end{code}" `isPrefixOf` l = ([],ls)
                      | otherwise                    = let (lns,r) = codelines ls
                                                       in ((n,l):lns,r)
 
+codeLines [] = error "Unterminated literate Code block"
+codeLines ((n,l):ls) | "\\end{Code}" `isPrefixOf` l = ([],ls)
+                     | otherwise                    = let (lns,r) = codeLines ls
+                                                      in ((n,l):lns,r)
+
 getName l = case r of
    ('}':_) -> nm
    _       -> error $ "missing '}' in \\IN"
